/*
 * sample-echo.c: Implements a Com::AtenaCom::Sample::Echo server
 *
 * Author:
 *   Miguel de Icaza  (miguel@helixcode.com)
 *
 * Modified by:
 *   Arturo Tena <arturo@directmail.org>
 */

#include <bonobo.h>

/*
 * This pulls the CORBA definitions for the Com::AtenaCom::Sample::Echo
 * server
 */
#include "Echo.h"

/*
 * This pulls the definition for the BonoboObject (Gtk Type)
 */
#include "sample-echo.h"

/*
 * A pointer to our parent object class
 */
static BonoboObjectClass *sample_echo_parent_class;

/*
 * The VEPV for the Com::AtenaCom::Sample::Echo objecg
 */
static POA_Com_AtenaCom_Sample_Echo__vepv sample_echo_vepv;

/*
 * Implemented GtkObject::destroy
 */
static void
sample_echo_object_destroy (GtkObject *object)
{
	SampleEcho *echo = SAMPLE_ECHO (object);

	g_free (echo->instance_data);

	GTK_OBJECT_CLASS (sample_echo_parent_class)->destroy (object);
}

void
impl_bonobo_propertybag_get_prop (BonoboPropertyBag *bag,
				  BonoboArg         *arg,
				  guint              arg_id,
				  gpointer           user_data)
{
	/* do nothing by now */
}

void
impl_bonobo_propertybag_set_prop (BonoboPropertyBag *bag,
				  const BonoboArg   *arg,
				  guint              arg_id,
				  gpointer           user_data)
{
	/* do nothing by now */
}

/*
 * CORBA Com::AtenaCom::Sample::Echo::echo method implementation
 */
static void
impl_sample_echo_echo (PortableServer_Servant  servant,
		       const CORBA_char       *string,
		       CORBA_Environment      *ev)
{
	SampleEcho *echo = SAMPLE_ECHO (bonobo_object_from_servant (servant));

	printf ("Echo message received: %s (echo instance data: %s)\n", string,
		echo->instance_data);
}

POA_Com_AtenaCom_Sample_Echo__epv *
sample_echo_get_epv (void)
{
	POA_Com_AtenaCom_Sample_Echo__epv *epv;

	epv = g_new0 (POA_Com_AtenaCom_Sample_Echo__epv, 1);

	/*
	 * This is the method invoked by CORBA
	 */
	epv->echo = impl_sample_echo_echo;

	return epv;
}

static void
init_echo_corba_class (void)
{
	sample_echo_vepv.Bonobo_Unknown_epv                = bonobo_object_get_epv ();
	sample_echo_vepv.Com_AtenaCom_Sample_Echo_epv   = sample_echo_get_epv ();
}

static void
sample_echo_class_init (SampleEchoClass *klass)
{
	GtkObjectClass *object_class = (GtkObjectClass *) klass;

	sample_echo_parent_class = gtk_type_class (bonobo_object_get_type ());

	object_class->destroy = sample_echo_object_destroy;

	init_echo_corba_class ();
}

GtkType
sample_echo_get_type (void)
{
	static GtkType type = 0;

	if (!type){
		GtkTypeInfo info = {
			"Echo",
			sizeof (SampleEcho),
			sizeof (SampleEchoClass),
			(GtkClassInitFunc) sample_echo_class_init,
			(GtkObjectInitFunc) NULL,
			NULL, /* reserved 1 */
			NULL, /* reserved 2 */
			(GtkClassInitFunc) NULL
		};

		type = gtk_type_unique (bonobo_object_get_type (), &info);
	}

	return type;
}

SampleEcho *
sample_echo_construct (SampleEcho *echo, Com_AtenaCom_Sample_Echo corba_echo)
{
	BonoboPropertyBag *property_bag;
	static int         i;
	
	g_return_val_if_fail (echo != NULL, NULL);
	g_return_val_if_fail (SAMPLE_IS_ECHO (echo), NULL);
	g_return_val_if_fail (corba_echo != CORBA_OBJECT_NIL, NULL);

	/*
	 * Call parent constructor
	 */
	if (!bonobo_object_construct (BONOBO_OBJECT (echo), (CORBA_Object) corba_echo))
		return NULL;

	/*
	 * Initialize echo
	 */
	echo->instance_data = g_strdup_printf ("Hello %d!", i++);
	
	/*
	 * Create and register Bonobo::PropertyBag interface
	 */
	property_bag = bonobo_property_bag_new (impl_bonobo_propertybag_get_prop,
						impl_bonobo_propertybag_set_prop,
						echo);
	if (!property_bag) {
		bonobo_object_unref (BONOBO_OBJECT (echo));
		return NULL;
	}
	bonobo_object_add_interface (BONOBO_OBJECT (echo),
				     BONOBO_OBJECT (property_bag));
	
	/*
	 * Success: return the GtkType we were given
	 */
	return echo;
}

/*
 * This routine creates the ORBit CORBA server and initializes the
 * CORBA side of things
 */
static Com_AtenaCom_Sample_Echo
create_corba_echo (BonoboObject *echo)
{
	POA_Com_AtenaCom_Sample_Echo *servant;
	CORBA_Environment                ev;

	servant = (POA_Com_AtenaCom_Sample_Echo *) g_new0 (BonoboObjectServant, 1);
	servant->vepv = &sample_echo_vepv;

	CORBA_exception_init (&ev);
	POA_Com_AtenaCom_Sample_Echo__init ((PortableServer_Servant) servant, &ev);
	ORBIT_OBJECT_KEY(servant->_private)->object = NULL;

	if (ev._major != CORBA_NO_EXCEPTION){
		g_free (servant);
		CORBA_exception_free (&ev);
		return CORBA_OBJECT_NIL;
	}

	CORBA_exception_free (&ev);

	/*
	 * Activates the CORBA object & binds to the servant
	 */
	return (Com_AtenaCom_Sample_Echo) bonobo_object_activate_servant (echo, servant);
}

SampleEcho *
sample_echo_new (void)
{
	SampleEcho                  *echo;
	Com_AtenaCom_Sample_Echo  corba_echo;

	echo = gtk_type_new (sample_echo_get_type ());

	corba_echo = create_corba_echo (BONOBO_OBJECT (echo));

	if (corba_echo == CORBA_OBJECT_NIL) {
		bonobo_object_unref (BONOBO_OBJECT (echo));
		return NULL;
	}
	
	return sample_echo_construct (echo, corba_echo);
}
