/*
 * test-sample-echo.c: Sample user for the Com::AtenaCom::Sample::Echo Bonobo component
 *
 * Author:
 *   Miguel de Icaza  (miguel@helixcode.com)
 *
 * Modified by:
 *   Arturo Tena <arturo@directmail.org>
 */


#include <gnome.h>
#include <liboaf/liboaf.h>
#include <bonobo.h>
#include "Echo.h"

static void
init_bonobo (int argc, char *argv [])
{
	CORBA_ORB orb;

        gnome_init_with_popt_table (
		"Com::AtenaCom::Sample::Echo client", "1.0",
		argc, argv,
		oaf_popt_options, 0, NULL); 

	orb = oaf_init (argc, argv);

	if (!bonobo_init (orb, CORBA_OBJECT_NIL, CORBA_OBJECT_NIL))
		g_error ("I could not initialize Bonobo");

	/*
	 * Enable CORBA/Bonobo to start processing requests
	 */
	bonobo_activate ();
}

/*
 * In this example, is not necesary to get a BonoboObjectClient
 * and then the CORBA reference to the interface, since
 * bonobo_get_object give us the CORBA reference to the interface
 * easily
 */

int 
main (int argc, char *argv [])
{
	Bonobo_Unknown               server;
	CORBA_Environment            ev;
	char                        *obj_id;
	char                        *obj_interface;

	init_bonobo (argc, argv);

	obj_id = "OAFIID:Com_AtenaCom_Sample_Echo";
	obj_interface = "Com/AtenaCom/Sample/Echo";

	CORBA_exception_init (&ev);

	/*
	 * Get a CORBA object reference to the interface, doing moniker stuff
	 */ 
	server = bonobo_get_object (obj_id, obj_interface, &ev);

	if (!server) {
		printf ("Could not create an instance of the %s component",
			obj_id);
		return 1;
	}

	/*
	 * Send a message
	 */
	Com_AtenaCom_Sample_Echo_echo
		(server, "This is the message from the client", &ev);

	/*
	 * We could query another interface on server and use it
	 */ 

	printf ("Press <ENTER> to exit... ");
	getchar ();

	/*
	 * Unref the CORBA reference to the interface
	 */
	Bonobo_Unknown_unref (server, &ev);

	CORBA_exception_free (&ev);

	return 0;
}
